From 156be401c38f7884ed65641aa202b72b698f3e91 Mon Sep 17 00:00:00 2001
From: Peter Geoghegan <pg@bowt.ie>
Date: Tue, 31 Jul 2018 18:33:30 -0700
Subject: [PATCH v2] Add table relcache invalidation to index builds.

It's necessary to make sure that owning tables have a relcache
invalidation prior to advancing the command counter to make
newly-entered catalog tuples for the index visible.  inval.c must be
able to roll back the local caches in the event of transaction abort.
There is only a problem when CREATE INDEX transactions abort, since
there is a generic invalidation once we reach index_update_stats().

This bug is of long standing.  Problems were made much more likely by
the addition of parallel CREATE INDEX (commit 9da0cc35284), but it is
strongly suspected that similar problems can be triggered without
involving plan_create_index_workers().  (plan_create_index_workers()
triggers a relcache build or rebuild, which previously only happened in
rare edge cases.)

Author: Peter Geoghegan
Reported-By: Luca Ferrari
Diagnosed-By: Andres Freund
Reviewed-By: Andres Freund
Discussion: https://postgr.es/m/CAKoxK+5fVodiCtMsXKV_1YAKXbzwSfp7DgDqUmcUAzeAhf=HEQ@mail.gmail.com
Backpatch: 9.3-
---
 src/backend/catalog/index.c | 6 ++++++
 1 file changed, 6 insertions(+)

diff --git a/src/backend/catalog/index.c b/src/backend/catalog/index.c
index 8b276bc430..2dad7b059e 100644
--- a/src/backend/catalog/index.c
+++ b/src/backend/catalog/index.c
@@ -978,6 +978,12 @@ index_create(Relation heapRelation,
 						!concurrent && !invalid,
 						!concurrent);
 
+	/*
+	 * Register relcache invalidation on the indexes' heap relation, to
+	 * maintain consistency of its index list
+	 */
+	CacheInvalidateRelcache(heapRelation);
+
 	/* update pg_inherits, if needed */
 	if (OidIsValid(parentIndexRelid))
 		StoreSingleInheritance(indexRelationId, parentIndexRelid, 1);
-- 
2.17.1

